
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.012.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "A Perl/Tk GUI builder in pure Perl/Tk. ",
  "dist_author" => [
    "Ala Qumsieh, faraco <skelic3\@gmail.com>"
  ],
  "dist_name" => "ZooZ",
  "dist_version" => "1.9.1",
  "license" => "perl",
  "module_name" => "ZooZ",
  "recursive_test_files" => 1,
  "requires" => {
    "Data::Dumper" => 0,
    "Getopt::Long" => 0,
    "Storable" => 0,
    "Tie::Watch" => 0,
    "Tk" => "804.025",
    "Tk::Adjuster" => 0,
    "Tk::BrowseEntry" => 0,
    "Tk::Compound" => 0,
    "Tk::Dialog" => 0,
    "Tk::DialogBox" => 0,
    "Tk::Font" => 0,
    "Tk::ItemStyle" => 0,
    "Tk::Labelframe" => 0,
    "Tk::NoteBook" => 0,
    "Tk::PNG" => 0,
    "Tk::Pane" => 0,
    "Tk::ProgressBar" => 0,
    "Tk::ROText" => 0,
    "Tk::ToolBar" => "0.12",
    "Tk::Tree" => 0,
    "base" => 0,
    "constant" => 0,
    "strict" => 0,
    "warnings" => 0
  },
  "script_files" => [
    "bin/zooz"
  ]
);


my %fallback_build_requires = (
  "Module::Build" => "0.28"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
