use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'bin/zooz',
    'lib/ZooZ.pm',
    'lib/ZooZ/Base.pm',
    'lib/ZooZ/Callbacks.pm',
    'lib/ZooZ/DefaultArgs.pm',
    'lib/ZooZ/Fonts.pm',
    'lib/ZooZ/Forms.pm',
    'lib/ZooZ/Generic.pm',
    'lib/ZooZ/Options.pm',
    'lib/ZooZ/Project.pm',
    'lib/ZooZ/TiedVar.pm',
    'lib/ZooZ/varRefs.pm'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
